package com.simonstl.fragment;

import java.util.HashMap;
import java.util.Iterator;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.Attributes;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The AttCollection class extends HashMap with functions for accepting and producing SAX AttributesImpl objects</p>
*
*
* <p>Version 0.03 adds <code>toString()</code> and <code>print()</code> methods.</p>
*
* <p>Version 0.02 adds the <code>cleanDuplicate()</code> method, which returns a fresh set of attributes.</p>
*
* @version 0.03 4 July 2001
* @author Simon St.Laurent
*/

public class AttCollection extends HashMap {

/**
* <p>The addAttributes method accepts a SAX AttributesImpl instance,
* breaks it into individual attributes, and adds them to the HashMap.</p>
*/
public void addAttributes(Attributes atts, Element parent) {
    String nsURI;
    String localName;
    String QName;
    String prefix;
    String content;
    String type;

    int numAtts=atts.getLength();
    for (int i=0; i<numAtts; i++) {
     nsURI=atts.getURI(i);
     localName=atts.getLocalName(i);
     QName=atts.getQName(i);
     if (localName==QName) {
       prefix="";
     } else {
       int stop=QName.length()-localName.length()-1;
       prefix=QName.substring(0,stop);
     }
     content=atts.getValue(i);
     type=atts.getType(i);
     Attribute newAtt=new Attribute(nsURI, localName, prefix, type, content);
     newAtt.setParentElement(parent);
//newAtt.print();
     this.put(newAtt.getKey(), newAtt);
    } //end for
}


/**
* <p>The replaceAttributes method accepts a SAX Attributes instance,
* breaks it into individual attributes, and adds them to a fresh HashMap.</p>
*/

public void replaceAttributes(Attributes atts, Element parent) {
     this.clear();
     this.addAttributes(atts, parent);
}
/**
* <p>The getAttributes method returns a SAX Attributes instance,
* built out of the Attributes which were stored in the HashMap.</p>
*/
public Attributes getAttributes() {
//System.out.println("Getting attributes");
  String QName;
  AttributesImpl atts= new AttributesImpl();
  Iterator it=this.keySet().iterator();
  while (it.hasNext()) {
//System.out.println("iterating");
    String key=(String) it.next();
//System.out.println("Got key:" +key);

    Attribute att=(Attribute)this.get(key);
//att.print();
    if (!(att.getPrefix().equals(""))) {
        QName=att.getPrefix()+":"+att.getLocalName();
    } else {
        QName=att.getLocalName();
    }
    atts.addAttribute(att.getNsURI(), att.getLocalName(), QName, att.getType(), att.getContent());
   }//end while

  return atts;
}

public AttCollection cleanDuplicate() {
   AttCollection clean=new AttCollection();
   
  Iterator it=this.keySet().iterator();
   while (it.hasNext()) {
    String key=(String) it.next();

    Attribute att=(Attribute)this.get(key);
    Attribute newAtt=(Attribute) att.cleanDuplicate();
    clean.put(newAtt.getKey(), newAtt);
   }//end while



   return clean;
}

public String toString() {
  String collectionOfAtts="";
  Iterator it=this.keySet().iterator();
   while (it.hasNext()) {
    String key=(String) it.next();

    Attribute att=(Attribute)this.get(key);
    collectionOfAtts=collectionOfAtts+att.toString();
   }//end while
  return collectionOfAtts;
}

public void print() {
  System.out.println(toString());
}

}