package com.simonstl.fragment;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The attribute class stores information about an attribute.</p>
*
* <p>Version 0.04 removes the <code>isAttribute</code> property.</p>
*
* <p>Version 0.03 adds improved <code>print() and <code>toString()</code> support.</p>
*
* <p>Version 0.02 supports the <code>cleanDuplicate()</code> method to avoid shallow copying issues.</p>
*
* <p>Version 0.01 is based on the <code>Skip</code> and <code>Element</code> classes.</p>
*
* @version 0.03 4 July 2001
* @author Simon St.Laurent
*/

public class Attribute extends DocComponent {
     protected String type="CDATA";

/**
* Attribute classes have NS names, and sometimes content.  This constructor sets all at once.
**/
public Attribute (String _nsURI, String _localName, String _prefix, String _type, String _content) {
    nsURI=_nsURI;
    localName=_localName;
    prefix=_prefix;
    type=_type;
    content=_content;
}

/**
* Attribute classes have NS names, and sometimes content.  This constructor waits for content.
**/
public Attribute (String _nsURI, String _localName, String _prefix) {
    nsURI=_nsURI;
    localName=_localName;
    prefix=_prefix;
}


/**
* Creates a new attribute from an old DocComponent.  
**/
public Attribute (DocComponent oldDocComponent) {
    nsURI=oldDocComponent.getNsURI();
    localName=oldDocComponent.getLocalName();
    prefix=oldDocComponent.getPrefix();
    content=oldDocComponent.getContent();
}
/**
* Attribute classes have NS names, and sometimes content.  This constructor waits for all.
**/
protected Attribute () {
}

public boolean matchComponent(DocComponent compare) {
    boolean returnValue=false;
    if ((compare instanceof Attribute) && (getKey().equals(compare.getKey()))) {
       returnValue=true;
    }
    return returnValue;
}


public boolean matchComponent (String match) {
    return (match.equals(getKey()));
}

/**
* Attributes return a key prefaced by @.
**/
public String getKey() {
    return ("@"+nsURI+">"+localName);
}




public void eventComponent(FragmentFilter handler) throws SAXException {
  //only appear in elements
}
public DocComponent cleanDuplicate() {
    Attribute clean=new Attribute();
if(getNsURI() !=null) 
    {clean.setNsURI (new String(getNsURI()));}
if (getLocalName() !=null)
    {clean.setLocalName (new String(getLocalName()));}
if (getPrefix() !=null)
    {clean.setPrefix (new String(getPrefix()));}
if (getBefore() !=null)
    {clean.setBefore (new String(getBefore()));}
if (getAfter() !=null)
    {clean.setAfter (new String(getAfter()));}
if (getContent() !=null)
    {clean.setContent(new String(getContent()));}

    return clean;
}


public void print() {
System.out.println(toString());
}

public String toString() {
   return ("@result: "+prefix+"("+nsURI+"):"+localName+" contains: " +("c: "+before+"{content}"+after+" "));
}

}