package com.simonstl.fragment;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLFilter;
import org.xml.sax.helpers.XMLFilterImpl;
import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The Element class stores information about elements (and perhaps someday attributes)
* in a format that supports namespaces, allows the preservation of prefix information,
* and supports matching on namespace URIs and localNames.</p>
*
* <p>I may eventually add more support for attributes, and also for wildcard matching on nsURI
* and/or localName.</p>
*
* <p>Version 0.11 fixed a bug in <code>addComponent()</code> that kept attribute rules from functioning.</p>
*
* <p>Version 0.10 adds improved printing support.</p>
*
* <p>Version 0.09 supports the cleanDuplicate() method to avoid shallow copying issues.</p>
*
* <p>Version 0.08 allows Elements to store parent and child information for the recording phase.</p>
*
* <p>Version 0.07 adds additional support for attributes through the attColl AttributeCollection, replacing the AttributesImpl I'd used before.  It's more overhead, but it gives me much finer-grained control over attributes.</p>
*
* <p>Version 0.06 extends DocComponent rather than trying to handle all cases itself.</p>
*
* <p>Version 0.05 adds support for <code>beforeInside</code> and <code>afterInside</code> attributes, 
* allowing rules to add content inside of the element they create.</p>
*
* <p>Version 0.04 changed support for skipping, allowing for the <code>chars</code> 
* element to operate when character output is desired.  It also now calls the startChars() 
* and endChars() methods of {@link com.simonstl.fragment.FragmentFilter FragmentFilter} to 
* give these characters their own place in the stack.  Finally, it now supports 
* <code>before</code> and <code>after</code> content.</p>
*
* <p>Version 0.03 added properties for content and attributes.  It also added the eventElement()
* method, which allows Element objects to write out their own SAX events.  This should make
* it easier to manage configuration in the future.</p>
*
* <p>version 0.02 added the getKey() method, so that Elements could define their own keys.
* This should permit future versions to support more complex matching than just nsURI>localName.
* (Making that work will involve revisions to matchElement (String match) as well.)</p>
*
* @version 0.09 4 July 2001
* @author Simon St.Laurent
*/

public class Element extends DocComponent {
    private String beforeInside=null;
    private String afterInside=null;
    private AttCollection attColl=new AttCollection();
    private LinkedList elementContent=new LinkedList();

public Element (String _nsURI, String _localName) {
    setNsURI(_nsURI);
    setLocalName(_localName);
    setPrefix("");
}

public Element (String _nsURI, String _localName, String _prefix) {
    setNsURI(_nsURI);
    setLocalName(_localName);
    setPrefix(_prefix);
}

public Element (String _nsURI, String _localName, String _prefix, Attributes _atts) {
    setNsURI(_nsURI);
    setLocalName(_localName);
    setPrefix(_prefix);
    setAtts(_atts);
}
public Element (String _nsURI, String _localName, String _prefix, String _content) {
    setNsURI(_nsURI);
    setLocalName(_localName);
    setPrefix(_prefix);
    setContent(_content);
}

protected Element(){
}

public Element (Element oldElement) {
    setNsURI (oldElement.getNsURI());
    setLocalName (oldElement.getLocalName());
    setPrefix (oldElement.getPrefix());
    setBefore(oldElement.getBefore());
    setAfter (oldElement.getAfter());
    setBeforeInside (oldElement.getBeforeInside());
    setAfterInside (oldElement.getAfterInside());
    attColl=new AttCollection();
    elementContent=new LinkedList();    

}

public DocComponent cleanDuplicate() {
    Element clean=new Element();
if(getNsURI() !=null) 
    {clean.setNsURI (new String(getNsURI()));}
if (getLocalName() !=null)
    {clean.setLocalName (new String(getLocalName()));}
if (getPrefix() !=null)
    {clean.setPrefix (new String(getPrefix()));}
if (getBefore() !=null)
    {clean.setBefore (new String(getBefore()));}
if (getAfter() !=null)
    {clean.setAfter (new String(getAfter()));}
if (getBeforeInside() !=null)
    {clean.setBeforeInside (new String(getBeforeInside()));}
if (getAfterInside() !=null)
    {clean.setAfterInside (new String(getAfterInside()));}
    clean.attColl=attColl.cleanDuplicate();
    clean.elementContent=new LinkedList();    

    return clean;

}

public boolean matchComponent(DocComponent compare) {
    boolean returnValue=false;
    if ((compare instanceof Element) && (getKey().equals(compare.getKey()))) {
       returnValue=true;
    }
    return returnValue;
}

public boolean matchComponent (String match) {
    return (match.equals(nsURI+">"+localName));
}

public String getKey() {
    return nsURI+">"+localName;
}

public void setAtts(Attributes newAtts) {
      attColl.replaceAttributes(newAtts, this);
}

public void addAtt(Attribute newAtt) {
//System.out.println("Adding an attribute.");
//newAtt.print();
      newAtt.setParentElement(this);
      attColl.put(newAtt.getKey(), newAtt);
//System.out.println("Added an attribute - key is "+newAtt.getKey());
//System.out.println("Now have "+attColl.size()+" atts.");
}


public Attributes getAtts() {
      return attColl.getAttributes();
}


public void setBeforeInside(String newBeforeInside) {
      beforeInside=newBeforeInside;
}

public String getBeforeInside() {
      return beforeInside;
}
public void setAfterInside(String newAfterInside) {
      after=newAfterInside;
}

public String getAfterInside() {
      return afterInside;
}
/**
* Sets the content for this component. Need to override DocComponent.
* Use only with simple types
**/
public void setContent(String _content) {
//System.out.println("adding characters!");
      char[] charArray=_content.toCharArray();
      String newContent=new String(charArray);
      Characters chars=new Characters(newContent);
//System.out.println(chars.toString());
      clearContent();
      elementContent.addFirst(chars);
//System.out.println("size: "+elementContent.size());

}

/**
* Returns the content for this component. Need to override DocComponent.
**/
public String getContent() {
//System.out.println("output size: "+elementContent.size());
  String content="";
  int pieces=elementContent.size();
  if (pieces>0) {
      Characters chars=(Characters) elementContent.getLast();
      content=chars.getContent();
  }
  return content;
}
/**
* Clears the content for this component. 
**/
public void clearContent() {
      elementContent.clear();
}


/**
* Adds content to this component. Need to override DocComponent.
**/
public void addContent(DocComponent newComponent) {
      if (newComponent instanceof Attribute) {
//System.out.println("adding attribute");
        addAtt ((Attribute) newComponent);
     } else {
//System.out.println("adding component");
//newComponent.print();
        elementContent.addLast(newComponent);
//System.out.println(elementContent);
     }
}



public void eventComponent(FragmentFilter handler) throws SAXException {
      String QName;
//System.out.println("skip status: "+skip);
      AttributesImpl atts=(AttributesImpl) attColl.getAttributes();
//      AttributesImpl atts=new AttributesImpl();

       if (!(prefix.equals(""))) {
        QName=prefix+":"+localName;
       } else {
        QName=localName;
       }
//if there's content for before the element, present it
      if (before!=null) {
        handler.freeCharacters(before.toCharArray(), 0, before.length());
      }

//if the element has a name, start it.
      if (!(( nsURI.equals("") ) && ( localName.equals("") ) && ( QName.equals("") ))) {
        handler.startFreeElement(nsURI, localName, QName, atts);
//System.out.println("starting element " + QName);
      } else {

      }

//if there's content for after the start tag but before the content, present it

      if (beforeInside!=null) {
        handler.freeCharacters(beforeInside.toCharArray(), 0, beforeInside.length());
      }

//--------------------------------------------------

//if the element has content, present it
      if(elementContent.size()>0) {
        eventElementContent(handler);
      }
//--------------------------------------------------
//if there's content for after the content but before the end tag, present it

      if (afterInside!=null) {
        handler.freeCharacters(afterInside.toCharArray(), 0, afterInside.length());
      }
//end the element
      if (!(( nsURI.equals("") ) && ( localName.equals("") ) && ( QName.equals("") ))) {
        handler.endFreeElement(nsURI, localName, QName);
//System.out.println("ending element " + QName);
      } else {

      }
//if there's content for after the element, present it
      if (after!=null) {
        handler.characters(after.toCharArray(), 0, after.length());
      }

}

private void eventElementContent (FragmentFilter handler) throws SAXException {
  int pieces=elementContent.size();
if (pieces>0) {
  for (int i=0; i<pieces; i++) {
//System.out.println(i+" of "+pieces+" pieces.");
    DocComponent currentComponent=(DocComponent) elementContent.get(i);
//currentComponent.print();
    currentComponent.eventComponent(handler);
  }    
}
}

private String toStringElementContent () {
  int pieces=elementContent.size();
  String value="";
  if (pieces>0) {
    for (int i=0; i<pieces; i++) {
      DocComponent currentComponent=(DocComponent) elementContent.get(i);
      value=value+" "+currentComponent.toString();
    }
  }
return value;
}

public void processComponentContent (FragmentRules rules) {
  int pieces=elementContent.size();
//System.out.println(elementContent); 


if (pieces>0) {
  for (int i=0; i<pieces; i++) {
    DocComponent currentComponent=(DocComponent) elementContent.get(i);
    rules.processComponent(currentComponent);
  }   
}
}


public void print() {
System.out.println(toString());
}

public String toString() {
   return (prefix+"("+nsURI+"):"+localName+" {"+toStringElementContent()+"} atts: "+attColl.toString()+"content: "+toStringElementContent());
}

}