package com.simonstl.fragment;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.IOException;
import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class loads the information stored in a rules document for use in fragment
* filtering.  It's based on Ethan Cerami's Xerces Quickstart, available at:</p>
* <p><a href="http://ecerami.com/xerces/" target="_top">http://ecerami.com/xerces/</a></p>
*
* <p>Future versions will remove the explicit reliance on the Xerces parser.</p>
*
*<p>Version 0.10 changed split="true|false" to type="match|split|series".</p>
*
*<p>Version 0.09 adds support for attributes in target elements.</p>
*
*<p>Version 0.08 adds support for attributes in result elements.</p>
*
* <p>Version 0.07 supports the move to {@link com.simonstl.fragment.DocComponent DocComponent} from {@link com.simonstl.fragment.Element Element}.  
* Note the use of the {@link com.simonstl.fragment.Characters Characters} and {@link com.simonstl.fragment.Skip Skip}
* classes.  Also note that this move is currently partial - no support yet for non-Element targets.</p>
*
* <p>Version 0.06 added support for the <code>beforeInside</code> and <code>afterInside</code> attributes
* of the <code>resultElement</code> element.</p>
*
* <p>Version 0.05 added support for the <code>chars</code> element 
* and the <code>before</code> and <code>after</code> attributes,
* and modified how support for the <code>skip</code> element
* worked.  Elements with empty <code>nsURI</code> and <code>localName</code> attributes
* are now treated as <code>chars</code> elements, not 
* <code>skip</code> elements.</p>
*
* <p>Version 0.04 added support for the <code>pattern</code> attribute, replacing <code>matchPattern</code>, 
* and added support for the <code>split</code> attribute
* of the <code>fragmentRule</code> element.</p>
*
* <p>Version 0.03 added support for the <code>skipFirst</code> and <code>repeat</code> attributes
* of the <code>fragmentRule</code> element.</p>
*
* <p>Version 0.02 added support for the <code>skip</code> element, which permits rules to skip portions
* of a match. (By default the first match is skipped.) </p> 
*
* @version 0.09 5 July 2001
* @author Simon St.Laurent
*/

public class RulesLoader extends DefaultHandler {
    
    public FragmentRules rules;   //set of rules
    private FragmentRule newRule; //current rule
    protected static ArrayList appliesTo;
    private Element parentElement=null;
    private boolean inAppliesTo=true;
    final int MATCH=0;
    final int SPLIT=1;
    final int SERIES=2;

    // Constructor
    public RulesLoader (String xmlFile, XMLReader parser) {
        
        //  Set Content Handler
        parser.setContentHandler (this);

        // Create a rules container
        rules=new FragmentRules();

 
        //  Parse the Document      
        try {
            parser.parse(xmlFile);
        } catch (SAXException e) {
            System.err.println (e);
        } catch (IOException e) {
            System.err.println (e);
        }
    }
    
    //  Start Element Event Handler
    public void startElement (String uri, String local,
        String qName, Attributes atts)  {

       
        if (uri.equals("http://simonstl.com/ns/fragments/")) {
//System.out.println(qName);
           if (local.equals("fragmentRule")) {
              String matchPattern=atts.getValue("pattern");
//System.out.println("pattern is:"+matchPattern);
              newRule=new FragmentRule(matchPattern);
              appliesTo=new ArrayList();
              String repeater=atts.getValue("repeat");
              if (repeater!=null){
//System.out.println("live repeater!");
                if (repeater.equals("true")) {
                  newRule.setRepeat(true);
//System.out.println("set repeat to true!");
                }
              }
              String skipFirst=atts.getValue("skipFirst");
              if (skipFirst!=null){
//System.out.println("live skip!");
               if (skipFirst.equals("false")) {
                  newRule.setSkipFirst(false);
//System.out.println("set skipFirst to false!");
                }
              }
              String isSplit=atts.getValue("type");
              if (isSplit!=null){
               if (isSplit.equals("match")) {
                  newRule.setIsSplit(MATCH);
               } else if (isSplit.equals("split")) {
                  newRule.setIsSplit(SPLIT);
               } else if (isSplit.equals("series"))
                  newRule.setIsSplit(SERIES);
              }

              //initialize Regular Expression
              newRule.complete();

           }

           if (local.equals("appliesTo")) {
             inAppliesTo=true;
           }
           if (local.equals("produce")) {
             inAppliesTo=false;
           }

           if (local.equals("element")) {
              String nsURI=atts.getValue("nsURI");
              String localName=atts.getValue("localName");
              String prefix=atts.getValue("prefix");

//System.out.println("result: "+prefix+"("+nsURI+"):"+localName);
              Element newElement=new Element(nsURI, localName, prefix);
		  newElement.setBefore(atts.getValue("before"));
              newElement.setAfter(atts.getValue("after"));
		  newElement.setBeforeInside(atts.getValue("beforeInside"));
              newElement.setAfterInside(atts.getValue("afterInside"));
              parentElement=newElement;

              if (inAppliesTo) {
              appliesTo.add(newElement);
              } else {
              newRule.addResult(newElement);
              }
           }


           if (local.equals("attribute")) {
              String nsURI=atts.getValue("nsURI");
              String localName=atts.getValue("localName");
              String prefix=atts.getValue("prefix");
              String type=atts.getValue("type");
              if (type==null) {type="CDATA";}
              String content=atts.getValue("content");
              Attribute newAtt=new Attribute(nsURI, localName, prefix, type, content);
              
              if (parentElement==null) {
                 if (inAppliesTo) {
                   appliesTo.add(newAtt);
                 } else {
                   newRule.addResult(newAtt);
                 }
              } else {
                parentElement.addAtt(newAtt);
              }
           }
           if (local.equals("skip")) {
//System.out.println("skip");
              Skip newSkip=new Skip();
		  newSkip.setBefore(atts.getValue("before"));
              newSkip.setAfter(atts.getValue("after"));
              newRule.addResult(newSkip);
           }

           if (local.equals("chars")) {
//System.out.println("chars");
              String before=atts.getValue("before");
		  String after=atts.getValue("after");
              Characters newCharacters=new Characters();
		  newCharacters.setBefore(before);
              newCharacters.setAfter(after);
              newRule.addResult(newCharacters);
//newCharacters.print();
           }

	  }
    }

    public void endElement (String uri, String local,
        String qName)  {

        if (uri.equals("http://simonstl.com/ns/fragments/")) {
//System.out.println(qName + " ended!");
           if (local.equals("element")) {
           parentElement=null;
           }
           if (local.equals("fragmentRule")) {
           inAppliesTo=true;  
              //time to clean up all the fragment rule list
//newRule.print();
              int size=appliesTo.size();
              for (int i=0; i<size; i++){
               DocComponent currentComponent=(DocComponent)appliesTo.get(i);
               String currentComponentName=currentComponent.toString(); 
//System.out.println("applies to: " + currentComponentName);
               rules.addRule(currentComponent, newRule);
              }
           }
	  }
    }

   
}